//Functions for managing the main application cycle, registering the callbacks
 // to create an application and its completion.
#include <app.h>

// Functions for displaying messages in the logging window.
#include <dlog.h>

// Functions for creating and managing user interface elements. Here are all the widgets,
// such as: windows, buttons, etc.
#include <Elementary.h>

// Functions for getting information about installed applications
#include <app_manager.h>

//Functions for using callback for hardware Back button and for
// using extended widgets for a circle screen
#include <efl_extension.h>

// Functions for getting information about installed packages
#include <package_manager.h>

#define FREE_RESOURCE(resource) \
   if (resource)                \
     free(resource);

typedef struct _UIData {
   Evas_Object *win;
   Evas_Object *conform;
   Evas_Object *rotary_selector;

   Eina_List *list_package;
} UIData;

typedef struct _Application_Info {
   char *label;
   char *path_icon;
   char *app_id;
} Application_Info;

static void
_win_back_cb(void *data, Evas_Object *obj, void *event_info)
{
   UIData *ui = data;

   elm_win_lower(ui->win);
}

static void
_item_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
   dlog_print(DLOG_INFO, LOG_TAG, "Element is pressed");

   UIData *ui = (UIData *)data;

   Eext_Object_Item *item = eext_rotary_selector_selected_item_get(ui->rotary_selector);
   Application_Info *application_info = item->data;

   app_control_h a_control;

   app_control_create(&a_control);
   app_control_set_app_id(a_control, application_info->app_id);
   if (app_control_send_launch_request(a_control, NULL, NULL) != APP_CONTROL_ERROR_NONE)
     dlog_print(DLOG_INFO, LOG_TAG, "Application running error");

   app_control_destroy(a_control);

   dlog_print(DLOG_INFO, LOG_TAG, "Application ID: %s", application_info->app_id);
}

static void
_rotary_selector_create(UIData *ui)
{
   ui->rotary_selector = eext_rotary_selector_add(ui->conform);
   eext_rotary_object_event_activated_set(ui->rotary_selector, EINA_TRUE);

   Eina_List *iterator        = NULL;
   Application_Info *app_info = NULL;

   //  Creating the elements of the rotary selector, based on the installed packages
   EINA_LIST_FOREACH(ui->list_package, iterator, app_info)
     {
        // Creating a rotary selector element
        Eext_Object_Item *item = eext_rotary_selector_item_append(ui->rotary_selector);
        item->data = app_info;

        // Creating an icon for an item
        Evas_Object *icon = elm_image_add(ui->rotary_selector);
        elm_image_file_set(icon, app_info->path_icon, NULL);
        evas_object_show(icon);

        // Set the icon and text into the element
        eext_rotary_selector_item_part_content_set(item,
                                                   "item,bg_image",
                                                   EEXT_ROTARY_SELECTOR_ITEM_STATE_NORMAL,
                                                   icon);

        eext_rotary_selector_item_part_text_set(item,
                                                "selector,main_text",
                                                app_info->label);
     }

   evas_object_smart_callback_add(ui->rotary_selector,
                                  "item,clicked",
                                  _item_clicked_cb,
                                  ui);

   elm_object_content_set(ui->conform, ui->rotary_selector);
   evas_object_show(ui->rotary_selector);
}

static void
_conformant_create(UIData *ui)
{
   ui->conform = elm_conformant_add(ui->win);

   //Specify to conformant occupies all allocated to him space
   // (horizontally and vertically)
   evas_object_size_hint_weight_set(ui->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

   _rotary_selector_create(ui);

   evas_object_show(ui->conform);
}

static void
_window_create(UIData *ui)
{
	// Create and configure the main window
   ui->win = elm_win_util_standard_add(NULL, NULL);

   // Create a conformant - in this case, the main container in the application,
   // which is also the interface between the application and
   // system elements, such as a keyboard.
   _conformant_create(ui);

   //  Set the size of the conformant widget the same as the size of the window
   elm_win_resize_object_add(ui->win, ui->conform);

   //Register the function that will be called
     //when you press the hardware Back button
   eext_object_event_callback_add(ui->win, EEXT_CALLBACK_BACK, _win_back_cb, ui);

   // Always display the window only after the entire base
   // user interface will be displayed
   evas_object_show(ui->win);
}

static bool
_app_manager_app_info_cb(app_info_h app_info, void *user_data)
{
   UIData *ui = user_data;
   Application_Info *application_info = calloc(1, sizeof(Application_Info));
   if (!application_info) return false;

   memset(application_info, 0x0, sizeof(Application_Info));

   app_info_get_app_id(app_info, &(application_info->app_id));
   app_info_get_label(app_info, &(application_info->label));
   app_info_get_icon(app_info, &(application_info->path_icon));

   bool is_nodisplay = false;
   app_info_is_nodisplay(app_info, &is_nodisplay);
   if (!is_nodisplay)
     ui->list_package = eina_list_append(ui->list_package, application_info);

   char *str_app_info = NULL;
   app_info_get_type(app_info, &str_app_info);
   dlog_print(DLOG_INFO, "lesson26", "Application type: %s", str_app_info);
   free(str_app_info);

   str_app_info = NULL;
   app_info_get_package(app_info, &str_app_info);
   dlog_print(DLOG_INFO, "lesson26", " Package ID: %s", str_app_info);
   free(str_app_info);

   dlog_print(DLOG_INFO, "lesson26", "Name: %s", application_info->label);
   dlog_print(DLOG_INFO, "lesson26", "Path to the icon: %s", application_info->path_icon);
   dlog_print(DLOG_INFO, "lesson26", "Application ID: %s", application_info->app_id);

   return true;
}

static bool
_app_create_cb(void *data)
{
   UIData *ui = data;

   srand(time(NULL));

   app_manager_foreach_app_info(_app_manager_app_info_cb, ui);

   _window_create(ui);

   return true;
}

static void
_app_terminate_cb(void *data)
{
   UIData *ui = data;

   //Before closing the application, delete the main widget (window),
    // then all  "children" widgets of this window (all widgets of this application) will be deleted.
   evas_object_del(ui->win);

   Application_Info *app_info = NULL;
   EINA_LIST_FREE(ui->list_package, app_info)
     {
        FREE_RESOURCE(app_info->label);
        FREE_RESOURCE(app_info->path_icon);
        FREE_RESOURCE(app_info->app_id);
        FREE_RESOURCE(app_info);
     }
}

int
main(int argc, char *argv[])
{
   UIData ui = {0,};
   ui_app_lifecycle_callback_s lifecycle_callbacks = {0,};

   //Register created functions to transfer them to the main loop of
   // this application.
   lifecycle_callbacks.create    = _app_create_cb;
   lifecycle_callbacks.terminate = _app_terminate_cb;

   // Calling the next function, launch the main application cycle
   return ui_app_main(argc, argv, &lifecycle_callbacks, &ui);
}
